-- #1
local average_orientation_frame_limit = 22 -- frames
-- the smaller this is the heavier you have to turn in 1 go to toggle drift... when this is bigger you can make multiple small turns and it will start drifting...
-- #2
local drift_min_orientation = 70 -- average orientation (angle in time [frame_limit])
-- the bigger this is - ... - right now when you make multiple small turns to make a big curve you do not trigger drift...
-- so reduce this and increase logging-time #1

-- to #1 and #2: they are combinatory so first is the time and second the amount you have to turn...

local drift_min_speed = 0.23
local drift_min_duration = 10 -- frames, how long drift will continue even when no longer triggering drift...

local monitored_cars = {}
script.on_event(defines.events.on_built_entity, function(event)
    local LuaEntity = event.created_entity
    if LuaEntity.name == "car" then
        table.insert(monitored_cars, {
            ["entity"] = LuaEntity,
            ["belts"] = {},
            ["last_orientation"] = LuaEntity.orientation,
            ["average_orientation_values"] = {}
        })
    end
end)
script.on_event(defines.events.on_tick, function(event)
    local tick = event.tick
    for _, car in pairs(monitored_cars) do
        if #car.belts ~= 0 then
            for _, belt in pairs(car.belts) do
                belt.destroy()
            end
        end
        local orientation = car.entity.orientation
        local orientation_delta = math.deg(math.sin((orientation - car.last_orientation) * math.pi * 2))
        car.last_orientation = orientation
        for i = #car.average_orientation_values, 1, -1 do
            if not ((i + 1) > average_orientation_frame_limit) then
                car.average_orientation_values[i + 1] = car.average_orientation_values[i]
            end
        end
        car.average_orientation_values[1] = math.abs(orientation_delta)
        local average_orientation = 0
        for i, value in pairs(car.average_orientation_values) do
            average_orientation = average_orientation + value
        end
        for i = 1, #car.average_orientation_values do
            average_orientation = average_orientation + car.average_orientation_values[i]
        end

        local turn_direction
        if orientation_delta > 0 then
            turn_direction = "right"
        elseif orientation_delta < 0 then
            turn_direction = "left"
        end
        -- store the last turn direction, so we can extend drifting into a certain direction even when you are driving straight again... the momentum still continues to drag into the counter direction to the last turn...
        if turn_direction then
            car.last_turn_direction = turn_direction
        end
        local direction
        if orientation >= 1 / 8 * 7 or orientation < 1 / 8 * 1 then
            direction = 0 -- N
        elseif orientation >= 1 / 8 * 1 and orientation < 1 / 8 * 3 then
            direction = 2 -- E
        elseif orientation >= 1 / 8 * 3 and orientation < 1 / 8 * 5 then
            direction = 4 -- S
        elseif orientation >= 1 / 8 * 5 and orientation < 1 / 8 * 7 then
            direction = 6 -- W
        end
        local speed = car.entity.speed
        local belt_direction
        if car.last_turn_direction == "right" and speed > 0 or car.last_turn_direction == "left" and speed < 0 then
            belt_direction = direction - 2
            if belt_direction < 0 then
                belt_direction = 6
            end
        elseif car.last_turn_direction == "left" and speed > 0 or car.last_turn_direction == "right" and speed < 0 then
            belt_direction = direction + 2
            if belt_direction > 6 then
                belt_direction = 0
            end
        end
        -- so better would be to only add values to average_orientation if we are already past the speed check
        -- now you could drive very slow, make turns then accelerate and it will trigger drift
        if speed >= drift_min_speed and average_orientation >= drift_min_orientation then
            -- game.print("toggle drift " .. tick)
            car.drift = true
            car.drift_start = tick
        end
        if car.drift_start then
            if car.drift_start + drift_min_duration < tick then
                car.drift = false
            end
        end
        if car.drift then
            if belt_direction then
                local LuaSurface = car.entity.surface
                local LuaEntity = {}
                if speed < 0.01 then
                    LuaEntity.name = "transport-belt"
                elseif speed < 0.05 then
                    LuaEntity.name = "fast-transport-belt"
                else
                    LuaEntity.name = "express-transport-belt"
                end
                LuaEntity.direction = belt_direction
                LuaEntity.position = car.entity.position
                if LuaSurface.can_place_entity(LuaEntity) then
                    table.insert(car.belts, LuaSurface.create_entity(LuaEntity))
                else
                    LuaEntity = nil
                end
            end
        end
        -- game.print(speed .. " " .. tick)
        -- game.print(average_orientation .. " " .. event.tick)
        -- game.print(tostring(turn_direction) .. " " .. tick)
    end
end)
function removeEntityFromMonitoring(entity)
    -- why would we still have an if check here - the function is called removeEntityFromMonitoring... - so if we cannot remove it from monitoring, then we should get an error...
    local LuaEntity = entity
    if LuaEntity.name == "car" then
        for i, car in pairs(monitored_cars) do
            if car.entity.unit_number == LuaEntity.unit_number then
                monitored_cars[i] = nil
            end
        end
    end
end
script.on_event(defines.events.on_player_mined_entity, function(event)
    removeEntityFromMonitoring(event.entity)
end)
script.on_event(defines.events.on_entity_died, function(event)
    removeEntityFromMonitoring(event.entity)
end)